# -*- coding: utf-8 -*-
"""navermovie_sentiment

Automatically generated by Colaboratory.

Original file is located at
    https://colab.research.google.com/drive/1GR7fuGEmr1JZTELs-6INad_SPFVsEaCN
"""

from google.colab import drive
drive.mount('/content/drive')

! ls "/content/drive/My Drive/word2vec"

!pip install konlpy

def read_data(filename):
    with open(filename, 'r') as f:
        data = [line.split('\t') for line in f.read().splitlines()]
        data = data[1:]   # header 제외
    return data
train_data = read_data('/content/drive/My Drive/word2vec/ratings_train.txt')
test_data = read_data('/content/drive/My Drive/word2vec/ratings_test.txt')

# row, column의 수가 제대로 읽혔는지 확인
print(len(train_data))     
print(len(train_data[0]))   
print(len(test_data))      
print(len(test_data[0]))

from konlpy.tag import Twitter
pos_tagger = Twitter()
def tokenize(doc):
    # norm, stem은 optional
    return ['/'.join(t) for t in pos_tagger.pos(doc, norm=True, stem=True)]
train_docs = [(tokenize(row[1]), row[2]) for row in train_data]
test_docs = [(tokenize(row[1]), row[2]) for row in test_data]
# 잘 들어갔는지 확인
from pprint import pprint
pprint(train_docs[0])

tokens = [t for d in train_docs for t in d[0]]
print(len(tokens))

import nltk
text = nltk.Text(tokens)
print(text)

print(len(tokens))
print(len(set(text.tokens)))
pprint(text.vocab().most_common(5))



#text.plot(50)

import nltk
nltk.download('stopwords')
text.collocations()

# 여기서는 최빈도 단어 2000개를 피쳐로 사용
# WARNING: 쉬운 이해를 위한 코드이며 time/memory efficient하지 않습니다
selected_words = [f[0] for f in text.vocab().most_common(2000)]
def term_exists(doc):
    return {'exists({})'.format(word): (word in set(doc)) for word in selected_words}
#  training corpus의 일부만 사용할 수 있음
train_docs = train_docs[:8000]
train_xy = [(term_exists(d), c) for d, c in train_docs]
test_xy = [(term_exists(d), c) for d, c in test_docs]

classifier = nltk.NaiveBayesClassifier.train(train_xy)
print(nltk.classify.accuracy(classifier, test_xy))

classifier.show_most_informative_features(10)

from collections import namedtuple
TaggedDocument = namedtuple('TaggedDocument', 'words tags')
# 여기서는 15만개 training documents 전부 사용함
# 8000개만 사용함
tagged_train_docs = [TaggedDocument(d, [c]) for d, c in train_docs]
tagged_test_docs = [TaggedDocument(d, [c]) for d, c in test_docs]

!pip install gensim
from gensim.models import doc2vec
#from gensim.models import Word2Vec
model = gensim.models.Doc2Vec(vector_size=300, window=10, min_count=5, workers=11,alpha=0.025, min_alpha=0.025, epoches=35)
#model = gensim.models.Word2Vec(vector_size=300, window=10, min_count=5, workers=11,alpha=0.025, min_alpha=0.025, epoches=35) 
model.build_vocab(tagged_train_docs) 
model.train(tagged_train_docs, epochs=model.iter, total_examples=model.corpus_count)

print(model.most_similar('공포/Noun'))

print(model.most_similar('ㅋㅋ/KoreanParticle'))

print(model.most_similar('^^/Punctuation'))

print(model.most_similar('ㅎㅎㅎ/KoreanParticle'))

text.concordance('왕/Noun', lines=10)

train_x = [model.infer_vector(doc.words) for doc in tagged_train_docs]
train_y = [doc.tags[0] for doc in tagged_train_docs]
len(train_x)

len(train_x[0])

test_x = [model.infer_vector(doc.words) for doc in tagged_test_docs]
test_y = [doc.tags[0] for doc in tagged_test_docs]

len(test_x)

len(test_x[0])

#epoches = 30 , len(train_x) = 8000 , accuracy = 0.77014
#epoches = 35 , len(train_x) = 8000 , accuracy = 0.77084
from sklearn.linear_model import LogisticRegression
classifier = LogisticRegression(random_state=1234)
classifier.fit(train_x, train_y)
classifier.score(test_x, test_y)

"""### **LSTM(Long Short Term Memory)방법으로 감성 분류**"""

import pandas as pd

train_data= pd.read_table('/content/drive/My Drive/word2vec/ratings_train.txt')
test_data= pd.read_table('/content/drive/My Drive/word2vec/ratings_test.txt')
print(len(train_data))

train_data[:5]

print(len(test_data))

test_data[:5]

train_data['label'].value_counts().plot(kind='bar')

print(train_data.groupby('label').size().reset_index(name='count'))

print(train_data.isnull().values.any())

print(train_data.isnull().sum())

train_data.loc[train_data.document.isnull()]

train_data=train_data.dropna(how='any') # Null 값이 존재하는 행 제거
print(train_data.isnull().values.any()) # Null 값이 존재하는지 확인

train_data['document'] = train_data['document'].str.replace("[^ㄱ-ㅎㅏ-ㅣ가-힣 ]","")
train_data[:5]

stopwords=['의','가','이','은','들','는','좀','잘','걍','과','도','를','으로','자','에','와','한','하다']
from konlpy.tag import Okt
okt = Okt()
X_train=[]
for sentence in train_data['document']:
    temp_X = []
    temp_X=okt.morphs(sentence, stem=True) # 토큰화
    temp_X=[word for word in temp_X if not word in stopwords] # 불용어 제거
    X_train.append(temp_X)

print(X_train[:3])

test_data=test_data.dropna(how='any') # Null 값 제거
test_data['document'] = test_data['document'].str.replace("[^ㄱ-ㅎㅏ-ㅣ가-힣 ]","") # 정규 표현식 수행

X_test=[]
for sentence in test_data['document']:
    temp_X = []
    temp_X=okt.morphs(sentence, stem=True) # 토큰화
    temp_X=[word for word in temp_X if not word in stopwords] # 불용어 제거
    X_test.append(temp_X)

from tensorflow.keras.preprocessing.text import Tokenizer
import numpy as np
from tensorflow.keras.preprocessing.sequence import pad_sequences
max_words = 35000
tokenizer = Tokenizer(num_words=max_words) # 상위 35,000개의 단어만 보존

tokenizer.fit_on_texts(X_train)
X_train = tokenizer.texts_to_sequences(X_train)
X_test = tokenizer.texts_to_sequences(X_test)

print(X_train[:3])

import matplotlib.pyplot as plt
print('리뷰의 최대 길이 :',max(len(l) for l in X_train))
print('리뷰의 평균 길이 :',sum(map(len, X_train))/len(X_train))
plt.hist([len(s) for s in X_train], bins=50)
plt.xlabel('length of Data')
plt.ylabel('number of Data')
plt.show()

max_len=30
# 전체 데이터의 길이는 30으로 맞춘다. ##
X_train = pad_sequences(X_train, maxlen=max_len)
X_test = pad_sequences(X_test, maxlen=max_len)

y_train=np.array(train_data['label'])
y_test=np.array(test_data['label'])

from tensorflow.keras.layers import Embedding, Dense, LSTM
from tensorflow.keras.models import Sequential

model = Sequential()
model.add(Embedding(max_words, 100))
model.add(LSTM(128))
model.add(Dense(1, activation='sigmoid'))
model.compile(optimizer='adam', loss='binary_crossentropy', metrics=['acc'])
history = model.fit(X_train, y_train, epochs=4, batch_size=60, validation_split=0.2) ##epoches 값 증가,

print("테스트 정확도: %.4f" % (model.evaluate(X_test, y_test)[1]))

"""ModelCheckpoint"""

from keras.callbacks import ModelCheckpoint
MODEL_SAVE_FOLDER_PATH = '/content/drive/My Drive/word2vec/model'
model_path = MODEL_SAVE_FOLDER_PATH + '{epochs : 02d}_{val_loss:.4f}.hdf5'
checkpoint = ModelCheckpoint(filepath = model_path,monitor='val_loss',verbose=1, save_best_only= True)

model = Sequential()
model.add(Embedding(max_words, 100))
model.add(LSTM(128))
model.add(Dense(1, activation='sigmoid'))
model.compile(optimizer='adam', loss='binary_crossentropy', metrics=['acc'])

model.fit(X_train, y_train, epochs=4, batch_size=60, validation_split=0.2, verbose=0, callbacks=[checkpoint])

print("테스트 정확도: %.4f" % (model.evaluate(X_test, y_test)[1]))

from keras.callbacks import EarlyStopping
early_stopping = EarlyStopping(monitor='val_loss', patience=100)
model.fit(X, Y, validation_split=0.2, epochs=5000, batch_size=500,callbacks=[EarlyStopping])

print("테스트 정확도: %.4f" % (model.evaluate(X_test, y_test)[1]))